@@ -124,21 +124,16 @@ read_random_source( byte *buffer, size_t length, int level )
     int n;
     int warn=0;
 
-    if( level == 2 ) {
+    if( level >= 2 ) {
 	if( fd_random == -1 )
 	    fd_random = open_device( "/dev/random", 8 );
 	fd = fd_random;
     }
     else {
-	/* fixme: we should use a simpler one for level 0,
-	 * because reading from /dev/urandom removes entropy
-	 * and the next read on /dev/random may have to wait */
 	if( fd_urandom == -1 )
 	    fd_urandom = open_device( "/dev/urandom", 9 );
 	fd = fd_urandom;
     }
-
-
     do {
 	fd_set rfds;
 	struct timeval tv;
