@@ -81,7 +81,7 @@ complete_sig( PKT_signature *sig, PKT_secret_key *sk, MD_HANDLE md )
 {
     int rc=0;
 
-    if( !(rc=check_secret_key( sk )) )
+    if( !(rc=check_secret_key( sk, 0 )) )
 	rc = do_sign( sk, sig, md, 0 );
 
     /* fixme: should we check whether the signature is okay?
@@ -220,6 +220,9 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( !multifile )
 	iobuf_push_filter( inp, md_filter, &mfx );
 
+    if( detached && !encrypt && !opt.rfc1991 )
+	afx.what = 2;
+
     if( opt.armor && !outfile  )
 	iobuf_push_filter( out, armor_filter, &afx );
     else {
