@@ -26,94 +26,114 @@
 #define SIGS_PER_RECORD 	((TRUST_RECORD_LEN-10)/5)
 #define ITEMS_PER_HTBL_RECORD	((TRUST_RECORD_LEN-2)/4)
 #define ITEMS_PER_HLST_RECORD	((TRUST_RECORD_LEN-6)/5)
+#define ITEMS_PER_PREF_RECORD	(TRUST_RECORD_LEN-10)
+#if ITEMS_PER_PREF_RECORD % 2
+  #error ITEMS_PER_PREF_RECORD must be even
+#endif
 #define MAX_LIST_SIGS_DEPTH  20
 
 
 #define RECTYPE_VER  1
 #define RECTYPE_DIR  2
 #define RECTYPE_KEY  3
-#define RECTYPE_CTL  4
-#define RECTYPE_SIG  5
-#define RECTYPE_HTBL 6
-#define RECTYPE_HLST 7
-#define RECTYPE_UID  8
+#define RECTYPE_UID  4
+#define RECTYPE_PREF 5
+#define RECTYPE_SIG  6
+#define RECTYPE_CACH 9
+#define RECTYPE_HTBL 10
+#define RECTYPE_HLST 11
+
+
+#define DIRF_CHECKED  1 /* everything has been checked, the other bits are
+			   valid */
+#define DIRF_MISKEY   2 /* some keys are missing, so they could not be checked*/
+#define DIRF_ERROR    4 /* severe errors: the key is not valid for some reasons
+			   but we mark it to avoid duplicate checks */
+#define DIRF_REVOKED  8 /* the complete key has been revoked */
+
+#define KEYF_REVOKED DIRF_REVOKED   /* this key has been revoked
+				       (only useful on subkeys)*/
+#define UIDF_REVOKED DIRF_REVOKED   /* this user id has been revoked */
 
 
 struct trust_record {
     int  rectype;
+    struct trust_record *next;	/* help pointer to build lists in memory */
+    struct trust_record *help_pref;
+    int  mark;
+    ulong recnum;
     union {
-	struct {	    /* version record: */
-	    byte version;   /* should be 1 */
-	    ulong locked;    /* pid of process which holds a lock */
+	struct {	     /* version record: */
+	    byte version;    /* should be 2 */
 	    ulong created;   /* timestamp of trustdb creation  */
 	    ulong modified;  /* timestamp of last modification */
 	    ulong validated; /* timestamp of last validation   */
-	    byte marginals_needed;
-	    byte completes_needed;
-	    byte max_cert_depth;
+	    ulong keyhashtbl;
 	} ver;
 	struct {	    /* directory record */
-	    ulong local_id;
-	    u32  keyid[2];
-	    ulong keyrec;   /* recno of primary public key record */
-	    ulong ctlrec;   /* recno of control record */
-	    ulong sigrec;   /* recno of first signature record (osolete) */
-	    ulong uidrec;   /* recno of first user-id record */
-	    ulong link;     /* to next dir record */
-	    byte no_sigs;   /* does not have sigature and checked */
+	    ulong lid;
+	    ulong keylist;  /* List of keys (the first is the primary key)*/
+	    ulong uidlist;  /* list of uid records */
+	    ulong cacherec; /* the cache record */
+	    byte ownertrust;
+	    byte dirflags;
 	} dir;
 	struct {	    /* primary public key record */
-	    ulong owner;
-	    u32  keyid[2];
+	    ulong lid;
+	    ulong next;    /* next key */
+	    byte keyflags;
 	    byte pubkey_algo;
 	    byte fingerprint_len;
 	    byte fingerprint[20];
-	    byte ownertrust;
 	} key;
 	struct {	    /* user id reord */
-	    ulong owner;    /* point back to the directory record */
-	    ulong chain;    /* points to next user id record */
-	    byte  subtype;  /* must be 0 */
-	    byte  namehash[20]; /* ripemd hash of the username */
-	    byte ownertrust;
-	    u32  prefrec;   /* recno of reference record */
+	    ulong lid;	    /* point back to the directory record */
+	    ulong next;    /* points to next user id record */
+	    ulong prefrec;   /* recno of preference record */
+	    ulong siglist;   /* list of valid signatures (w/o self-sig)*/
+	    byte uidflags;
+	    byte namehash[20]; /* ripemd hash of the username */
 	} uid;
-	struct {	    /* control record */
-	    ulong owner;
-	    byte blockhash[20];
-	    byte trustlevel;   /* calculated trustlevel */
-	} ctl;
+	struct {	    /* preference record */
+	    ulong lid;	    /* point back to the directory record */
+			    /* or 0 for a glocal pref record */
+	    ulong next;    /* points to next pref record */
+	    byte  data[ITEMS_PER_PREF_RECORD];
+	} pref;
 	struct {	    /* signature record */
-	    ulong owner;  /* local_id of record owner (pubkey record) */
-	    ulong chain;  /* offset of next record or NULL for last one */
+	    ulong lid;
+	    ulong next;   /* recnno of next record or NULL for last one */
 	    struct {
-		ulong  local_id; /* of pubkey record of signator (0=unused) */
-		byte flag;     /* reserved */
+		ulong lid;	 /* of pubkey record of signator (0=unused) */
+		byte flag;	 /* SIGRF_xxxxx */
 	    } sig[SIGS_PER_RECORD];
 	} sig;
+	struct {	    /* cache record */
+	    ulong lid;
+	    byte blockhash[20];
+	    byte trustlevel;   /* calculated trustlevel */
+	} cache;
 	struct {
 	    ulong item[ITEMS_PER_HTBL_RECORD];
 	} htbl;
 	struct {
-	    ulong chain;
-	    struct {
-		byte hash;
-		ulong rnum;
-	    } item[ITEMS_PER_HLST_RECORD];
+	    ulong next;
+	    ulong rnum[ITEMS_PER_HLST_RECORD]; /* of a key record */
 	} hlst;
     } r;
 };
 typedef struct trust_record TRUSTREC;
 
 typedef struct {
-    ulong     local_id;    /* localid of the pubkey */
+    ulong     lid;	   /* localid */
     ulong     sigrec;
-    ulong     sig_id;	   /* returned signature id */
+    ulong     sig_lid;	   /* returned signatures LID */
     unsigned  sig_flag;    /* returned signature record flag */
     struct {		   /* internal data */
 	int init_done;
 	int eof;
 	TRUSTREC rec;
+	ulong nextuid;
 	int index;
     } ctl;
 } SIGREC_CONTEXT;
@@ -122,11 +142,15 @@ typedef struct {
 /*-- tdbio.c --*/
 int tdbio_set_dbname( const char *new_dbname, int create );
 const char *tdbio_get_dbname(void);
-void tdbio_dump_record( ulong rnum, TRUSTREC *rec, FILE *fp );
+void tdbio_dump_record( TRUSTREC *rec, FILE *fp );
 int tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected );
-int tdbio_write_record( ulong recnum, TRUSTREC *rec );
+int tdbio_write_record( TRUSTREC *rec );
+int tdbio_delete_record( ulong recnum );
 ulong tdbio_new_recnum(void);
-int tdbio_search_record( PKT_public_key *pk, TRUSTREC *rec );
+int tdbio_search_dir_bypk( PKT_public_key *pk, TRUSTREC *rec );
+int tdbio_search_dir_byfpr( const byte *fingerprint, size_t fingerlen,
+					int pubkey_algo, TRUSTREC *rec );
+int tdbio_delete_uidrec( ulong dirlid, ulong uidlid );
 
 
 #define buftoulong( p )  ((*(byte*)(p) << 24) | (*((byte*)(p)+1)<< 16) | \
