@@ -134,7 +134,8 @@ const char *print_fname_stdout( const char *s );
 /*-- miscutil.c --*/
 u32 make_timestamp(void);
 u32 add_days_to_timestamp( u32 stamp, u16 days );
-const char *strtimestamp( u32 stamp );
+const char *strtimestamp( u32 stamp ); /* GMT */
+const char *asctimestamp( u32 stamp ); /* localized */
 void print_string( FILE *fp, byte *p, size_t n, int delim );
 int answer_is_yes( const char *s );
 
